# Script is set up as follows: the NBD_dir and S_dir are directories in which the full CREST output folders are placed.
# The script searches for the .energies file inside a CREST output folder, and uses the number of lines to assess the
# number of generated conformers.

import os
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np

# Define directories
NBD_dir = './results/CREST_NBD'
S_dir = './results/CREST'

# List items in directories
nbd_items = os.listdir(NBD_dir)
s_items = os.listdir(S_dir)

# Create dictionary to store the count of conformers
conformer_count_dict = {}

# Initialize the dictionary structure
for root, dirs, files in os.walk(NBD_dir):
    for directory in dirs:
        conformer_count_dict[directory] = {'s maj 1': 0, 's maj 2': 0, 's min 1': 0, 's min 2': 0, 'nbd': 0}

# Count conformers for NBD directory
for root, dirs, files in os.walk(NBD_dir):
    for file in files:
        if file.endswith('.energies'):
            ligand = os.path.basename(root)
            with open(os.path.join(root, file)) as f:
                conformer_count = len(f.readlines())
                conformer_count_dict[ligand]['nbd'] = conformer_count

# Count conformers for S directory
for root, dirs, files in os.walk(S_dir):
    for file in files:
        if file.endswith('.energies'):
            ligand = os.path.basename(root).split('_')[0]
            with open(os.path.join(root, file)) as f:
                conf = len(f.readlines())
                if 'min' in root:
                    if'_2'in root:
                        conformer_count_dict[ligand]['s min 2'] = conf
                    else:
                        conformer_count_dict[ligand]['s min 1'] = conf
                else:
                    if '_2' in root:
                        conformer_count_dict[ligand]['s maj 2'] = conf
                    else:
                        conformer_count_dict[ligand]['s maj 1'] = conf

# Create a DataFrame
df = pd.DataFrame(conformer_count_dict).T.reset_index()

# Rename the 'index' column to 'ComplexName'
df = df.rename(columns={'index': 'ComplexName'})

# Mapping dictionary for ligand names (original ligand numbers from experiment to manuscript ligand numbers)
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
df['ComplexName'] = df['ComplexName'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Sort DataFrame based on ligand order
df = df.set_index('ComplexName').loc[ligand_order].reset_index()

#print(df)


################ Begin plotting part ##################
# Set the plot style
sns.set(style="whitegrid")
palette = sns.color_palette('Paired')
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

# Separate data
maj_df = df[['ComplexName', 's maj 1', 's maj 2']]
min_df = df[['ComplexName', 's min 1', 's min 2']]
nbd_df = df[['ComplexName', 'nbd']]

# Prepare x-positions
x = np.arange(len(df))

# Plot 'min_df' and 'maj_df' next to each other
fig, ax = plt.subplots(figsize=(10, 6))

ax.set_xticks(x)
ax.set_xticklabels(ligand_order, ha='right')

# Plot 'maj_df'
ax.bar(x, maj_df['s maj 1'], width=0.3, label='L-Rh-S major 1', color=palette[5])
ax.bar(x, maj_df['s maj 2'], width=0.3, bottom=maj_df['s maj 1'], label='L-Rh-S major 2', color=palette[4])

# Plot 'min_df' next to 'maj_df'
ax.bar(x - 0.3, min_df['s min 1'], width=0.3, label='L-Rh-S minor 1', color=palette[1])
ax.bar(x - 0.3, min_df['s min 2'], width=0.3, bottom=min_df['s min 1'], label='L-Rh-S minor 2', color=palette[0])

# Plot 'nbd_df' next to 'maj_df'
ax.bar(x + 0.3, nbd_df['nbd'], width=0.3, label='L-Rh-NBD', color=palette[3])

plt.xlabel('Ligand')
plt.ylabel('Count')
#plt.title('min_df and maj_df Data')

plt.legend()
plt.savefig('./plots/number_conformers.png', dpi=500, bbox_inches='tight')
plt.show()



