# Script is set up as follows: DFT_dir contains all the DFT-optimized .log and .xyz files.
# CREST_dir contains all the conformer .xyz files generated with CREST.
# dft_info_file is generated by get_gaussian_info.py to check for imaginary frequencies and extract the total energy.
# First, the DFT-optimized conformers are separated into groups based on the substrate coordination mode. For each
# Conformer the energy is found in dft_info_file, and within each group the lowest energy conformer is located.
# Within each group, all conformers, apart from the lowest energy conformer, are compared to the lowest energy conformer
# through an RMSD calculation. The data is placed in a csf file, with columns containing the lowest energy conformer,
# the conformer which is compared to the lowest energy conformer, and the calculated RMSD value.
# The same is done for the CREST .xyz files: group by substrate coordination mode, locate lowest energy conformer,
# calculate RMSD, create .csv file.
# Then, the energy from dft_info_file, the RMSD value after DFT, and RMSD value after CREST are placed together in a
#.csv file.
# The same procedure is repeated for the model-catalyst structures involving NBD.

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np
import os
import subprocess

############# Calculate RMSD after DFT ###########################
DFT_dir = './results/SM2_all'
CREST_dir = './results/Descriptors'
dft_info_file = 'L-Rh-S_info.csv'
dft_info_df = pd.read_csv(dft_info_file, delimiter=',')

dict = {}
conformer_dict = {}
rows = []

files = os.listdir(DFT_dir)
for file in files:
    filename = file.split('.')[0]
    if file.endswith('.xyz'):
        if 'conformer' in file:
            basename = file.split('_conformer')[0]

            if not basename in dict:
                dict[basename] = []

            dict[basename].append(filename)

# Iterate over the bases
for base, conformers in dict.items():
    lowest_energy = float('inf')
    lowest_energy_conformer = None

    # Iterate over conformers in current base
    for conformer in conformers:
        name = conformer.split('_DFT')[0]
        conformer_log = f'{name}.log'

        # Find the corresponding E in the DFT info file
        conformer_info = dft_info_df[dft_info_df['file'] == conformer_log]

        if not conformer_info.empty:
            energy = conformer_info.iloc[0]['E']
            if energy < lowest_energy:
                lowest_energy = energy
                lowest_energy_conformer = conformer


    # Calculate RMSD for the lowest energy conformer against all other conformers in the base
    lowest_energy_conformer_xyz = f'{lowest_energy_conformer}.xyz'
    lowest_energy_conformer_dir = os.path.join(DFT_dir, lowest_energy_conformer_xyz)
    for conformer in conformers:
        if conformer == lowest_energy_conformer:
            rows.append((lowest_energy_conformer, conformer, 0.0))
        elif conformer != lowest_energy_conformer:
            conformer_xyz = f'{conformer}.xyz'
            conformer_dir = os.path.join(DFT_dir, conformer_xyz)

            command = f'calculate_rmsd --no-hydrogen {lowest_energy_conformer_dir} {conformer_dir}'
            result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                    text=True)
            rmsd_value = float(result.stdout.split('\n')[0])

            rows.append((lowest_energy_conformer, conformer, rmsd_value))

# Create CSV file
rmsd_df = pd.DataFrame(rows, columns=['lowest_DFT', 'conformer_DFT', 'RMSD_value'])
sorted_df = rmsd_df.sort_values(by=['lowest_DFT', 'RMSD_value'])
sorted_df.to_csv('L-Rh-S_rmsd_after_dft.csv', index=False)

########### Calculate RMSD of CREST structures, before DFT ############################
# Iterate over the bases
for base, conformers in dict.items():
    lowest_energy = float('inf')
    lowest_energy_conformer = None

    # Iterate over conformers in current base
    for conformer in conformers:
        name = conformer.split('_DFT')[0]
        conformer_log = f'{name}.log'

        # Find the corresponding E in the DFT info file
        conformer_info = dft_info_df[dft_info_df['file'] == conformer_log]

        if not conformer_info.empty:
            energy = conformer_info.iloc[0]['E']
            if energy < lowest_energy:
                lowest_energy = energy
                lowest_energy_conformer = conformer

    # Calculate RMSD for the lowest energy conformer against all other conformers in the base
    lowest_energy_conformer_crest = lowest_energy_conformer.split('_DFT')[0]
    lowest_energy_conformer_crest_xyz = f'{lowest_energy_conformer_crest}.xyz'
    lowest_energy_conformer_crest_dir = os.path.join(CREST_dir, base, lowest_energy_conformer_crest_xyz)
    for conformer in conformers:
        conformer_crest = conformer.split('_DFT')[0]
        if conformer_crest == lowest_energy_conformer_crest:
            rows.append((lowest_energy_conformer_crest, conformer_crest, 0.0))
        elif conformer_crest != lowest_energy_conformer_crest:
            conformer_crest_xyz = f'{conformer_crest}.xyz'
            conformer_crest_dir = os.path.join(CREST_dir, base, conformer_crest_xyz)

            command = f'calculate_rmsd --no-hydrogen {lowest_energy_conformer_crest_dir} {conformer_crest_dir}'
            result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                    text=True)
            rmsd_value = float(result.stdout.split('\n')[0])

            rows.append((lowest_energy_conformer_crest, conformer_crest, rmsd_value))

# Create CSV file
rmsd_df = pd.DataFrame(rows, columns=['lowest_DFT', 'conformer_crest', 'RMSD_value_crest'])
sorted_df = rmsd_df.sort_values(by=['lowest_DFT', 'RMSD_value_crest'])
sorted_df.to_csv('L-Rh-S_rmsd.csv', index=False)

############# Create E vs RMSD file ##################
E_dft_df = pd.read_csv('L-Rh-S_info.csv')
RMSD_DFT = pd.read_csv('L-Rh-S_rmsd_after_dft.csv')
RMSD_DFT = RMSD_DFT[(RMSD_DFT['RMSD_value'] != 0.0)]
RMSD_crest = pd.read_csv('L-Rh-S_rmsd.csv')
RMSD_crest = RMSD_crest[(RMSD_crest['RMSD_value_crest'] != 0.0)]

# Create a new DataFrame to store the results
result_df = pd.DataFrame(columns=['ComplexName', 'Coordination', 'number', 'lowest_DFT', 'other', 'E_dif_(kJ/mol)', 'rmsd_value_DFT'])

# Iterate over rows in RMSD_DFT
for index, row in RMSD_DFT.iterrows():
    base_conformer = row['lowest_DFT']
    base_conformer = base_conformer.split('_DFT')[0]
    other_conformer = row['conformer_DFT']
    other_conformer = other_conformer.split('_DFT')[0]
    rmsd_value = row['RMSD_value']

    # Extract ligand, coordination, and distinguish between 1 and 2
    ligand = base_conformer.split('_')[0]
    coord = 'min' if 'min' in base_conformer else 'maj'
    number = '2' if '_2_' in base_conformer else '1'

    # Look up E values in E_dft_df
    base_conformer_e = E_dft_df.loc[E_dft_df['file'] == f"{base_conformer}.log", 'E'].values
    other_conformer_e = E_dft_df.loc[E_dft_df['file'] == f"{other_conformer}.log", 'E'].values

    # Check if the files are present in E_dft_df
    if not base_conformer_e or not other_conformer_e:
        continue  # Skip if file not found in E_dft_df

    # Extract the first element from the arrays
    base_conformer_e = base_conformer_e[0]
    other_conformer_e = other_conformer_e[0]

    # Calculate E difference in kJ/mol
    e_difference_kjmol = (other_conformer_e - base_conformer_e) * 2625.5  # 1 Hartree = 2625.5 kJ/mol

    # Look for RMSD value in RMSD_crest
    rmsd_value_crest = RMSD_crest.loc[RMSD_crest['conformer_crest'] == other_conformer, 'RMSD_value_crest'].values[0]

    # Append the results to result_df
    result_df = result_df.append({
        'ComplexName': ligand,
        'Coordination': coord,
        'number': number,
        'lowest_DFT': base_conformer,
        'other': other_conformer,
        'E_dif_(kJ/mol)': e_difference_kjmol,
        'rmsd_value_DFT': rmsd_value,
        'rmsd_value_crest': rmsd_value_crest
    }, ignore_index=True)

# Save the updated DataFrame to a new CSV file
result_df.to_csv('E_vs_RMSD_DFT_CREST.csv', index=False)

######################### Begin plotting part ###########################
# Mapping dictionary for ligand names, from original experiment names to manuscript names
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
result_df['ComplexName'] = result_df['ComplexName'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Sort DataFrame based on ligand order
result_df = result_df.set_index('ComplexName').loc[ligand_order].reset_index()

# Set the plot style
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

# Define colors for 'maj' and 'min'
colors = {'maj': sns.color_palette('tab10')[3], 'min': sns.color_palette('tab10')[0]}
# Define markers for '1' and '2'
markers = {'1': 'o', '2': 's'}

ligands = result_df['ComplexName'].unique()

# E vs RMSD after DFT
for ligand in ligands:
    ligand_data = result_df[result_df['ComplexName'] == ligand]

    plt.figure(figsize=(8, 6))
    sns.scatterplot(
        x='rmsd_value_DFT',
        y='E_dif_(kJ/mol)',
        data=ligand_data,
        hue='Coordination',
        style='number',  # 'number' is assumed to be the column containing '1' and '2'
        markers=['o', 's'],  # marker for '1' and '2'
        palette=colors,
        alpha=0.7
    )
    # Customize the plot
    plt.xlabel('RMSD (Å)')
    plt.ylabel('∆E_DFT (kJ/mol)')
   # plt.title(f'E vs RMSD for Ligand {ligand} DFT')
    plt.grid(True)

    custom_legend_items = [plt.Line2D([0], [0], marker=markers['2'], color=colors['maj'], lw=0),
                plt.Line2D([0], [0], marker=markers['1'], color=colors['maj'], lw=0),
                plt.Line2D([0], [0], marker=markers['2'], color=colors['min'], lw=0),
                plt.Line2D([0], [0], marker=markers['1'], color=colors['min'], lw=0)]
    plt.legend(custom_legend_items, ['L-Rh-S major 1', 'L-Rh-S major 2', 'L-Rh-S minor 1', 'L-Rh-S minor 2'], bbox_to_anchor=(-0.1, 1))

    # Show the plot
    plt.savefig(f'./plots/E_vs_RMSD_{ligand}_DFT.png', dpi=600, bbox_inches='tight')
    plt.show()

# E vs RMSD after CREST, before DFT
for ligand in ligand_order:
    ligand_data = result_df[result_df['ComplexName'] == ligand]

    plt.figure(figsize=(8, 6))
    sns.scatterplot(
        x='rmsd_value_crest',
        y='E_dif_(kJ/mol)',
        data=ligand_data,
        hue='Coordination',
        style='number',  # 'number' is assumed to be the column containing '1' and '2'
        markers=['o', 's'],  # marker for '1' and '2'
        palette=colors,
        alpha=0.7
    )
    # Customize the plot
    plt.xlabel('RMSD (Å)')
    plt.ylabel('∆E_DFT (kJ/mol)')
    #plt.title(f'E vs RMSD for Ligand {ligand} CREST')
    plt.grid(True)

    custom_legend_items = [plt.Line2D([0], [0], marker=markers['2'], color=colors['maj'], lw=0),
                           plt.Line2D([0], [0], marker=markers['1'], color=colors['maj'], lw=0),
                           plt.Line2D([0], [0], marker=markers['2'], color=colors['min'], lw=0),
                           plt.Line2D([0], [0], marker=markers['1'], color=colors['min'], lw=0)]
    plt.legend(custom_legend_items, ['S major 1', 'S major 2', 'S minor 1', 'S minor 2'], bbox_to_anchor=(-0.1, 1))

    # Show the plot
    plt.savefig(f'./plots/E_vs_RMSD_{ligand}_CREST.png', dpi=600, bbox_inches='tight')
    plt.show()


################################################################################
############# Everything repeated for L-Rh-NBD structures ######################

############## Calculate RMSD after DFT ###########################
DFT_dir = './results/NBD_all'
CREST_dir = './results/Descriptors_NBD'
dft_info_file = 'L-Rh-NBD_info.csv'
dft_info_df = pd.read_csv(dft_info_file, delimiter=',')

dict = {}
conformer_dict = {}
rows = []

files = os.listdir(DFT_dir)
for file in files:
    filename = file.split('.')[0]
    if file.endswith('.xyz'):
        if 'conformer' in file:
            basename = file.split('_conformer')[0]

            if not basename in dict:
                dict[basename] = []

            dict[basename].append(filename)

# Iterate over the bases
for base, conformers in dict.items():
    lowest_energy = float('inf')
    lowest_energy_conformer = None

    # Iterate over conformers in current base
    for conformer in conformers:
        name = conformer.split('_DFT')[0]
        conformer_log = f'{name}.log'

        # Find the corresponding E in the DFT info file
        conformer_info = dft_info_df[dft_info_df['file'] == conformer_log]

        if not conformer_info.empty:
            energy = conformer_info.iloc[0]['E']
            if energy < lowest_energy:
                lowest_energy = energy
                lowest_energy_conformer = conformer


    # Calculate RMSD for the lowest energy conformer against all other conformers in the base
    lowest_energy_conformer_xyz = f'{lowest_energy_conformer}.xyz'
    lowest_energy_conformer_dir = os.path.join(DFT_dir, lowest_energy_conformer_xyz)
    for conformer in conformers:
        if conformer == lowest_energy_conformer:
            rows.append((lowest_energy_conformer, conformer, 0.0))
        elif conformer != lowest_energy_conformer:
            conformer_xyz = f'{conformer}.xyz'
            conformer_dir = os.path.join(DFT_dir, conformer_xyz)

            command = f'calculate_rmsd --no-hydrogen {lowest_energy_conformer_dir} {conformer_dir}'
            result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                    text=True)
            rmsd_value = float(result.stdout.split('\n')[0])

            rows.append((lowest_energy_conformer, conformer, rmsd_value))

# Create CSV file
rmsd_df = pd.DataFrame(rows, columns=['lowest_DFT', 'conformer_DFT', 'RMSD_value'])
sorted_df = rmsd_df.sort_values(by=['lowest_DFT', 'RMSD_value'])
sorted_df.to_csv('L-Rh-NBD_rmsd_after_dft.csv', index=False)

########### Calculate RMSD of CREST structures, before DFT ############################
# Iterate over the bases
for base, conformers in dict.items():
    lowest_energy = float('inf')
    lowest_energy_conformer = None

    # Iterate over conformers in current base
    for conformer in conformers:
        name = conformer.split('_DFT')[0]
        conformer_log = f'{name}.log'

        # Find the corresponding E in the DFT info file
        conformer_info = dft_info_df[dft_info_df['file'] == conformer_log]

        if not conformer_info.empty:
            energy = conformer_info.iloc[0]['E']
            if energy < lowest_energy:
                lowest_energy = energy
                lowest_energy_conformer = conformer

    # Calculate RMSD for the lowest energy conformer against all other conformers in the base
    lowest_energy_conformer_crest = lowest_energy_conformer.split('_DFT')[0]
    lowest_energy_conformer_crest_xyz = f'{lowest_energy_conformer_crest}.xyz'
    lowest_energy_conformer_crest_dir = os.path.join(CREST_dir, base, lowest_energy_conformer_crest_xyz)
    for conformer in conformers:
        conformer_crest = conformer.split('_DFT')[0]
        if conformer_crest == lowest_energy_conformer_crest:
            rows.append((lowest_energy_conformer_crest, conformer_crest, 0.0))
        elif conformer_crest != lowest_energy_conformer_crest:
            conformer_crest_xyz = f'{conformer_crest}.xyz'
            conformer_crest_dir = os.path.join(CREST_dir, base, conformer_crest_xyz)

            command = f'calculate_rmsd --no-hydrogen {lowest_energy_conformer_crest_dir} {conformer_crest_dir}'
            result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                    text=True)
            rmsd_value = float(result.stdout.split('\n')[0])

            rows.append((lowest_energy_conformer_crest, conformer_crest, rmsd_value))

# Create CSV file
rmsd_df = pd.DataFrame(rows, columns=['lowest_DFT', 'conformer_crest', 'RMSD_value_crest'])
sorted_df = rmsd_df.sort_values(by=['lowest_DFT', 'RMSD_value_crest'])
sorted_df.to_csv('L-Rh-NBD_rmsd.csv', index=False)

### E vs RMSD
E_dft_df = pd.read_csv('L-Rh-NBD_info.csv')
RMSD_DFT = pd.read_csv('L-Rh-NBD_rmsd_after_dft.csv')
RMSD_DFT = RMSD_DFT[(RMSD_DFT['RMSD_value'] != 0.0)]
RMSD_crest = pd.read_csv('L-Rh-NBD_rmsd.csv')
RMSD_crest = RMSD_crest[(RMSD_crest['RMSD_value_crest'] != 0.0)]

# Create a new DataFrame to store the results
result_df = pd.DataFrame(columns=['ComplexName', 'lowest_DFT', 'other', 'E_dif_(kJ/mol)', 'rmsd_value_DFT'])

# Iterate over rows in RMSD_DFT
for index, row in RMSD_DFT.iterrows():
    base_conformer = row['lowest_DFT']
    base_conformer = base_conformer.split('_DFT')[0]
    other_conformer = row['conformer_DFT']
    other_conformer = other_conformer.split('_DFT')[0]
    rmsd_value = row['RMSD_value']

    # Extract ligand
    ligand = base_conformer.split('_')[0]

    # Look up E values in E_dft_df
    base_conformer_e = E_dft_df.loc[E_dft_df['file'] == f"{base_conformer}.log", 'E'].values
    other_conformer_e = E_dft_df.loc[E_dft_df['file'] == f"{other_conformer}.log", 'E'].values

    # Check if the files are present in E_dft_df
    if not base_conformer_e or not other_conformer_e:
        continue  # Skip if file not found in E_dft_df

    # Extract the first element from the arrays
    base_conformer_e = base_conformer_e[0]
    other_conformer_e = other_conformer_e[0]

    # Calculate E difference in kJ/mol
    e_difference_kjmol = (other_conformer_e - base_conformer_e) * 2625.5  # 1 Hartree = 2625.5 kJ/mol

    # Look for RMSD value in RMSD_crest
    rmsd_value_crest = RMSD_crest.loc[RMSD_crest['conformer_crest'] == other_conformer, 'RMSD_value_crest'].values[0]

    # Append the results to result_df
    result_df = result_df.append({
        'ComplexName': ligand,
        'lowest_DFT': base_conformer,
        'other': other_conformer,
        'E_dif_(kJ/mol)': e_difference_kjmol,
        'rmsd_value_DFT': rmsd_value,
        'rmsd_value_crest': rmsd_value_crest
    }, ignore_index=True)

# Save the updated DataFrame to a new CSV file
result_df.to_csv('L-Rh-NBD_E_vs_RMSD_DFT_CREST.csv', index=False)

######################### Begin plotting part ############################
# Mapping dictionary for ligand names, from original experimental names to manuscript names
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
result_df['ComplexName'] = result_df['ComplexName'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Set the plot style
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

# Define colors for 'maj' and 'min'
colors = sns.color_palette('tab10')[2]
darkgreen = (1/255, 25/255, 16/255)

ligands = result_df['ComplexName'].unique()

# Plot together E vs RMSD after DFT and E vs RMSD after CREST, indicated by color
for ligand in ligand_order:
    ligand_data = result_df[result_df['ComplexName'] == ligand]

    plt.figure(figsize=(8, 6))
    sns.scatterplot(x='rmsd_value_DFT', y='E_dif_(kJ/mol)', data=ligand_data, color='limegreen', alpha=0.7, label='DFT')
    sns.scatterplot(x='rmsd_value_crest', y='E_dif_(kJ/mol)', data=ligand_data, color=darkgreen, alpha=0.7, label='CREST')

    # Customize the plot
    plt.xlabel('RMSD (Å)')
    plt.ylabel('∆E_DFT (kJ/mol)')
    #plt.title(f'E vs RMSD for Ligand {ligand} together')
    plt.grid(True)
    plt.legend()

    # Show the plot
    plt.savefig(f'./plots/NBD_E_vs_RMSD_{ligand}_together.png', dpi=600, bbox_inches='tight')
    plt.show()





