#!/home/adarsh/anaconda3/envs/auto_func/bin/python
'''
Extracts key info from Gaussian Opt & Freq output files
Syntaxis:
    > python get_gaussian_info.py a1.log a2.log *.out output.csv
    - the last argument must be path to output table file
    - other paths must be Gaussian outputs
'''

#%% Imports

import sys, os, fnmatch, random
import pandas as pd
import re

#%% Functions

def IsFloat(value):
    try:
        float(value)
        return True
    except ValueError:
        return False


def GetGaussianInfo(path):
    '''
    Extracts info from Gaussian Opt & Freq output
    '''
    greetings = ['For the best Prof in the world :-*',
                 'This xyz file was prepared for the Hetman of SPB Host',
                 '*Regular sexist joke*',
                 'Science is interesting and if you don\'t agree you can fuck off',
                 'Two things are infinite: the universe and human stupidity; and I\'m not sure about the universe',
                 'Somewhere, something incredible is waiting to be known',
                 'Heard melodies are sweet, but those unheard, are sweeter',
                 'If I have seen further it is by standing on the shoulders of Giants',
                 'Life would be tragic if it weren\'t funny',
                 'Physics is like sex: sure, it may give some practical results, but that\'s not why we do it',
                 'Professor, aren\'t these the same questions as last year\'s final exam? Yes, but this year the answers are different',
                 'I don\'t want to believe. I want to know',
                 'I would rather have questions that can\'t be answered than answers that can\'t be questioned']
    info = {}
    with open(path, 'r') as inpf:
        text = [_.strip() for _ in inpf.readlines()]
    # is geometry optimized
    start = None
    for i, line in enumerate(text):
        if 'Optimization completed' in line:
            start = i
            break
    if start is None:
        # try to find last SCF energy
        s = None
        for i, line in enumerate(text):
            if 'SCF Done:' in line:
                s = i
        if s is None:
            return None
        info['opt'] = '-'
        info['E'] = [_ for _ in text[s].split() if IsFloat(_)][0]
        info['xyz'] = None
        info['NImag'] = '-'
        info['ZPE'] = '-'
        info['E_ZPE'] = '-'
        info['E_T'] = '-'
        info['H'] = '-'
        info['G'] = '-'
        return info
    
    # find coordinates
    info['opt'] = '+'
    for i, line in enumerate(text):
        if i < start:
            continue
        if line == 'Standard orientation:' or line == 'Input orientation:':
            s = i + 5
            break
    e = s + 1
    while '--------' not in text[e]:
        e += 1
    # extract coordinates
    xyz = []
    for line in text[s:e]:
        xyz.append( ' '.join(line.split()[-3:]) )
    # find atoms
    s = None
    for i, line in enumerate(text):
        if i < start:
            continue
        if 'Mulliken charges' in line:
            s = i + 2
            break
    e = s + 1
    while 'Sum' not in text[e]:
        e += 1
    # extract atoms
    atoms = []
    for line in text[s:e]:
        atoms.append(line.split()[1])
    # add to info
    xyz = [atom + ' ' + coords for atom, coords in zip(atoms, xyz)]
    xyz = [str(len(xyz)), random.choice(greetings)] + xyz
    info['xyz'] = '\n'.join(xyz) + '\n'
    
    # energy
    s = start
    while 'SCF Done:' not in text[s]:
        s -= 1
    info['E'] = [_ for _ in text[s].split() if IsFloat(_)][0]
    
    # imaginary frequencies
    freqs = []
    for i, line in enumerate(text):
        if i < start:
            continue
        if 'Frequencies --' in line:
            freqs += [float(_) for _ in line.split() if IsFloat(_)]
    if len(freqs) == 0:
        info['NImag'] = '-'
    else:
        info['NImag'] = len([_ for _ in freqs if _ < 0])
    
    # find all energies
    s = None
    for i, line in enumerate(text):
        if i < start:
            continue
        if 'Zero-point correction=' in line:
            s = i
            break
    if not s:
        info['ZPE'] = '-'
        info['E_ZPE'] = '-'
        info['E_T'] = '-'
        info['H'] = '-'
        info['G'] = '-'
        return info
    
    # parse energies
    info['ZPE'] = [_ for _ in text[s].split() if IsFloat(_)][0]
    info['E_ZPE'] = [_ for _ in text[s+4].split() if IsFloat(_)][0] if s+4 < len(text) else '-'
    info['E_T'] = [_ for _ in text[s+5].split() if IsFloat(_)][0] if s+5 < len(text) else '-'
    info['H'] = [_ for _ in text[s+6].split() if IsFloat(_)][0] if s+6 < len(text) else '-'
    info['G'] = [_ for _ in text[s+7].split() if IsFloat(_)][0] if s+7 < len(text) else '-'
    
    return info


def InfoToLine(info):
    '''
    Returns info as CSV line
    '''
    line = info['opt'] + ',' + info['E'] + ',' + str(info['NImag'])
    for key in ('ZPE', 'E_ZPE', 'E_T', 'H', 'G'):
        line += ',' + info[key]
    
    return line



#%% Main code

HELP = \
'''
Extracts key info from Gaussian Opt & Freq output files
Syntaxis:
    > python get_gaussian_info.py a1.log a2.log *.out output.csv
    - the last argument must be path to output table file
    - other paths must be Gaussian outputs
'''

if __name__ == '__main__':
    # get arguments
    args = sys.argv[1:]
    if len(args) == 0:
        print(HELP)
        sys.exit()
    if len(args) < 2:
        print('Error: there must be at least 2 arguments')
        sys.exit()
    input_folder, output_csv = args[:-1][0], args[-1]

    # check if the output_csv already exists and create a backup
    if os.path.exists(output_csv):
        backup_csv = output_csv + '.backup'
        os.rename(output_csv, backup_csv)
        print(f"Backup of existing {output_csv} created as {backup_csv}")

    # Retrieve all .log files in the input folder
    log_files = []
    for root, _, files in os.walk(input_folder):
        for file in files:
            if fnmatch.fnmatch(file, '*.log'):
                log_files.append(os.path.join(root, file))

    # Prepare the CSV header
    header = 'dir,file,opt,E,ImFreqs,ZPE,E_ZPE,E_T,H,G'
    N = header.count(',') - 2
    csv_data = [header]

    # Loop through each .log file and extract info
    for log_file in log_files:
        # Get info
        try:
            info = GetGaussianInfo(log_file)
        except (SystemExit, KeyboardInterrupt):
            raise
        except:
            info = None

        # Prepare CSV line
        folder, filename = os.path.split(log_file)
        line = folder + ',' + filename + ','
        if '.' in filename:
            name = '.'.join(filename.split('.')[:-1])
        else:
            name = filename
        line += InfoToLine(info) if info else ','.join(['-'] * (N + 1))
        csv_data.append(line)

        # Save XYZ
        if info and info['opt'] == '+':
            with open(os.path.join(folder, name + '_DFT.xyz'), 'w') as outf:
                outf.write(info['xyz'])

   # Define a custom sorting function to sort by the L number in the "file" column
    def sort_by_file(line):
        match = re.search(r'L(\d+)', line)  # Extract the number after "L"
        if match:
            return int(match.group(1))
        return 0  # Return 0 for lines that don't match the pattern

    # Sort the CSV data using the custom sorting function
    sorted_csv_data = sorted(csv_data[1:], key=sort_by_file)

    # Prepare the sorted CSV data with the header
    sorted_csv_data_with_header = [csv_data[0]] + sorted_csv_data

    # Save the naturally sorted extracted data to the output CSV
    with open(output_csv, 'w') as outf:
        outf.write('\n'.join(sorted_csv_data_with_header) + '\n')
