# Setup of script: the 'original' energy values, i.e., the energies of CREST input files, are first found.
# Per conformer, the difference between the original energy value and conformer energy value is calculated.
# This is done for L-Rh-S major and minor first, plotted, and then followed for L-Rh-NBD and plotted.

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

# .csv file of all optimized L-Rh-S CREST output conformers
S_all = pd.read_csv('L-Rh-S-conformer_info.csv')
# .csv file of all optimized L-Rh-S major and minor CREST input structures (i.e., "original" structures)
S_maj_orig = pd.read_csv('L-Rh-S-major_info.csv', delimiter=';')
S_min_orig = pd.read_csv('L-Rh-S-minor_info.csv', delimiter=';')
# .csv file of all optimized L-Rh-NBD CREST output conformers
NBD_all = pd.read_csv('.L-Rh-NBD-conformer_info.csv')
# .csv file of all optimized L-Rh-NBD CREST input structures (i.e., "original" structures)
NBD_orig = pd.read_csv('original_NBD.csv')

############# L-Rh-S major and minor #################################
# Filter on major and minor coordination
S_all_min = S_all[S_all['file'].str.contains('min')].copy()
S_all_maj = S_all[~S_all['file'].str.contains('min')].copy()

# Extract ComplexName from 'file' column for both DataFrames
S_all_min['Complex_coord'] = S_all_min['file'].apply(lambda x: x.split('_min')[0])
S_all_maj['Complex_coord'] = S_all_maj['file'].apply(lambda x: x.split('_conformer')[0])
S_min_orig['Complex_coord'] = S_min_orig['file'].apply(lambda x: x.split('.')[0])
S_maj_orig['Complex_coord'] = S_maj_orig['file'].apply(lambda x: x.split('.')[0])

# Get unique ligands with their substrate coordination mode
ligands = S_all_min['Complex_coord'].unique()

rows = []
for ligand in ligands:
    # For L-Rh-S minor
    row_in_S_min_orig = S_min_orig[(S_min_orig['Complex_coord'] == ligand)]
    orig_E_min = row_in_S_min_orig['E'].values[0]
    number = '2' if '_2' in ligand else '1'
    rows.append([ligand.split('_')[0], number, 'min', orig_E_min])
    # For L-Rh-S major
    row_in_S_maj_orig = S_maj_orig[(S_maj_orig['Complex_coord'] == ligand)]
    orig_E = row_in_S_maj_orig['E'].values[0]
    number = '2' if '_2' in ligand else '1'
    rows.append([ligand.split('_')[0], number, 'maj', orig_E])

df_S = pd.DataFrame(rows, columns=['ComplexName', 'number', 'coord', 'original_E_hartree'])
#df_SM2.to_csv('TRY.csv', index=False)

# Conformers compared to original
rows = []

for index, row in S_all.iterrows():
    filename = row['file'].split('_DFT')[0]
    complex_name = filename.split('_')[0]
    coord = 'min' if 'min' in filename else 'maj'
    number = '2' if '_2_' in filename else '1'
    conformer_E = row['E']

    # Find the original energy in DataFrame that we created above
    orig_row = df_S[
        (df_S['ComplexName'] == complex_name) & (df_S['coord'] == coord) & (df_S['number'] == number)]

    orig_E = orig_row['original_E_hartree'].values[0]

    # Stabilization energy
    dif = (conformer_E - orig_E) * 2625.49962

    rows.append([complex_name, coord, filename, conformer_E, orig_E, dif])

results_S = pd.DataFrame(rows, columns=['ComplexName', 'coord', 'conformer', 'conformer_E_hartree', 'original_E_hartree', 'dif_kJ/mol'])
results_S = results_S.sort_values(by=['ComplexName', 'coord'])
#results_SM2.to_csv('tryout')

# Mapping dictionary for ligand names, original experimental numbers to manuscript numbers)
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
results_S['ComplexName'] = results_S['ComplexName'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Sort DataFrame based on ligand order
results_S = results_S.set_index('ComplexName').loc[ligand_order].reset_index()

# Replace 'min' with 'L-Rh-S minor' and 'maj' with 'L-Rh-S major' for legend labels
results_S['coord'] = results_S['coord'].replace({'min': 'L-Rh-S minor', 'maj': 'L-Rh-S major'})

# Set the plot style
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

PROPS = {
    'boxprops':{'edgecolor':'black'},
    'medianprops':{'color':'black'},
    'whiskerprops':{'color':'black'},
    'capprops':{'color':'black'}}

palette = sns.color_palette('tab10')
plt.figure(figsize=(12, 6))
box_plot = sns.boxplot(x='ComplexName', y='dif_kJ/mol', data=results_S, hue='coord', hue_order=['L-Rh-S minor', 'L-Rh-S major'], palette=[palette[0], palette[3
]], width=0.75, **PROPS)
plt.xlabel('Ligand')
plt.ylabel('∆E (kJ/mol)')
#plt.title('E stabilization with CREST')
plt.grid(True)
plt.legend(loc='lower left')

plt.savefig('./plots/L-Rh-S_CREST_stabilization.png', dpi=500, bbox_inches='tight')
plt.show()


###########################################################################
################# Repeat for L-Rh-NBD structures ##########################
# Get ligands
NBD_all['ComplexName'] = NBD_all['file'].apply(lambda x: x.split('_')[0])
ligands = NBD_all['ComplexName'].unique()

rows = []
for ligand in ligands:
    filename = f'{ligand}.log'
    orig_row = NBD_orig[(NBD_orig['file'] == filename)]
    orig_E = orig_row['E'].values[0]
    rows.append([ligand, orig_E])
df_NBD = pd.DataFrame(rows, columns=['ComplexName', 'original_E_hartree'])

# Conformers compared to original
rows = []

for index, row in NBD_all.iterrows():
    filename = row['file'].split('_DFT')[0]
    complex_name = filename.split('_')[0]
    conformer_E = row['E']

    # Find the original energy
    orig_row = df_NBD[(df_NBD['ComplexName'] == complex_name)]
    orig_E = orig_row['original_E_hartree'].values[0]

    # Stabilization energy
    dif = (conformer_E - orig_E) * 2625.49962

    rows.append([complex_name, filename, conformer_E, orig_E, dif])
results_NBD = pd.DataFrame(rows, columns=['ComplexName', 'conformer', 'conformer_E_hartree', 'original_E_hartree', 'dif_kJ/mol'])
results_NBD = results_NBD.sort_values(by=['ComplexName'])

# Mapping dictionary for ligand names
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
results_NBD['ComplexName'] = results_NBD['ComplexName'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Sort DataFrame based on ligand order
results_NBD = results_NBD.set_index('ComplexName').loc[ligand_order].reset_index()

# Set the plot style
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})
PROPS = {
    'boxprops':{'edgecolor':'black'},
    'medianprops':{'color':'black'},
    'whiskerprops':{'color':'black'},
    'capprops':{'color':'black'}}

#data_crest_NBD = data_crest_NBD.sort_values(by='DeltaDeltaG_kJ/mol')
palette = sns.color_palette('tab10')
plt.figure(figsize=(8, 6))
box_plot = sns.boxplot(x='ComplexName', y='dif_kJ/mol', data=results_NBD, color=palette[2], width=0.75, **PROPS)
plt.xlabel('Ligand')
plt.ylabel('∆E (kJ/mol)')
#plt.title('E stabilization with CREST')
plt.grid(True)

plt.savefig('./plots/L-Rh-NBD_CREST_stabilization.png', dpi=500, bbox_inches='tight')
plt.show()
