# Setup of script: first, four functions are defined.
# The first one extracts the ligand part (ligand + metal center).
# The second function extracts the substrate part (substrate + metal center + ligand donor atoms).
# The third function searches for the lowest energy conformer and performs RMSD calculation on given .xyz file
# compared to the .xyz file of the lowest energy conformer.
# The last function computes statistics from the calculated RMSD values and returns a .csv file with mean and std
# per conformer ensemble, for the ligand and substrate separately.
# Below the definition of thefunctions, the functions are used on all DFT-optimized .xyz files of the conformers
# of L-Rh-S major and L-Rh-S minor, followed by L-Rh-NBD. Then the data is plotted as boxplots.

from morfeus.io import read_xyz, write_xyz
import os
from molecular_graph import MolGraph
import periodictable
import numpy as np
from molecular_graph import bfs
import pandas as pd
import subprocess
import matplotlib.pyplot as plt
import seaborn as sns

def save_only_metal_ligand_complex_in_xyz(input_xyz_file, output_directory):
    elements, coordinates = read_xyz(input_xyz_file)
    metal_centers = ['Rh']

    periodic_table = [element.symbol for element in periodictable.elements]

    ### Check if elements coming from morfeus are given as numbers.
    # If not, make conversion using the periodic table package.

    if str(elements[0]).isdigit():
        elements = list(elements)
        for elem_id, element in enumerate(elements):
            elements[elem_id] = periodic_table[element]
        elements = np.array(elements)

    ### Call to the MolGraph() functionality
    mg = MolGraph()
    mg.read_xyz_coord_from_mf(elements=elements, coordinates=coordinates)
    graph = mg.adj_list

    metal_center_id = None
    for elem_id, element in enumerate(elements):
        if element in metal_centers:
            metal_center_id = int(elem_id)
            break

    if metal_center_id is not None:
        metal_center_bonds_ = graph[metal_center_id]
        del graph[metal_center_id]

        for key, bonds_to_atom in zip(graph.keys(), graph.values()):
            graph[key] = list(bonds_to_atom)

            for bond_id, bond in enumerate(bonds_to_atom):
                if bond == metal_center_id:
                    graph[key].pop(bond_id)

        store_ligands = []

        for node in metal_center_bonds_:
            store_ligands.append(list(np.sort(bfs([], graph, node))))

        # Remove duplicates and store in new list --> clean_ligands
        clean_ligands = []

        for ligand in store_ligands:
            if ligand not in clean_ligands:
                clean_ligands.append(ligand)

        # Find the longest ligand and other ligand
        longest_ligand = max(clean_ligands, key=len)

        # Save indices of ligands and metal center
        ligands_indices = longest_ligand + [metal_center_id]
        #ligands_indices.append(metal_center_id)

        # Define the output file path and filename
        input_filename = os.path.basename(input_xyz_file)  # Extract the input file name
        output_filename = os.path.splitext(input_filename)[0] + '_ligand.xyz'  # Construct the new filename
        output_path = os.path.join(output_directory, output_filename)

        # Write elements and coordinates of ligands and metal center to the xyz file
        comments = [f"Metal-ligand complex extracted from {input_xyz_file}"]
        write_xyz(output_path, elements=elements[ligands_indices], coordinates=coordinates[ligands_indices],
                  comments=comments)

def save_shortest_ligand_complex_in_xyz(input_xyz_file, output_directory):
    elements, coordinates = read_xyz(input_xyz_file)
    metal_centers = ['Rh']

    periodic_table = [element.symbol for element in periodictable.elements]

    if str(elements[0]).isdigit():
        elements = list(elements)
        for elem_id, element in enumerate(elements):
            elements[elem_id] = periodic_table[element]
        elements = np.array(elements)

    mg = MolGraph()
    mg.read_xyz_coord_from_mf(elements=elements, coordinates=coordinates)
    graph = mg.adj_list

    metal_center_id = None
    for elem_id, element in enumerate(elements):
        if element in metal_centers:
            metal_center_id = int(elem_id)
            break

    if metal_center_id is not None:
        metal_center_bonds_ = graph[metal_center_id]
        del graph[metal_center_id]

        for key, bonds_to_atom in zip(graph.keys(), graph.values()):
            graph[key] = list(bonds_to_atom)

            for bond_id, bond in enumerate(bonds_to_atom):
                if bond == metal_center_id:
                    graph[key].pop(bond_id)

        store_ligands = []

        for node in metal_center_bonds_:
            store_ligands.append(list(np.sort(bfs([], graph, node))))

        # Remove duplicates and store in a new list --> clean_ligands
        clean_ligands = []

        for ligand in store_ligands:
            if ligand not in clean_ligands:
                clean_ligands.append(ligand)

        # Find the shortest ligand
        shortest_ligand = min(clean_ligands, key=len)

        # Find the 'P' atoms and their coordinates
        p_atoms_indices = [i for i, elem in enumerate(elements) if elem == 'P']
        p_atoms_coordinates = coordinates[p_atoms_indices]

        # Combine indices of the shortest ligand, 'P' atoms, and the metal center
        ligands_indices = shortest_ligand + p_atoms_indices + [metal_center_id]

        # Define the output file path and filename
        input_filename = os.path.basename(input_xyz_file)  # Extract the input file name
        output_filename = os.path.splitext(input_filename)[0] + '_substrate.xyz'  # Construct the new filename
        output_path = os.path.join(output_directory, output_filename)

        # Write elements and coordinates to the new XYZ file
        comments = [f"Shortest ligand complex extracted from {input_xyz_file}"]
        write_xyz(output_path, elements=elements[ligands_indices], coordinates=coordinates[ligands_indices],
                  comments=comments)

def calculate_rmsd_and_create_csv(base_dir, output_filename, dft_info_file, delim):
    dft_info_df = pd.read_csv(dft_info_file, delimiter=delim)

    dict = {}
    rows = []

    files = os.listdir(base_dir)
    for file in files:
        filename = file.split('.')[0]
        if file.endswith('.xyz'):
            if 'conformer' in file:
                basename = file.split('_conformer')[0]
            else:
                basename = file.split('_DFT')[0]

            if not basename in dict:
                dict[basename] = []

            dict[basename].append(filename)

    for base, conformers in dict.items():
        coord = 'min' if 'min' in base else 'maj'

        lowest_energy = float('inf')
        lowest_energy_conformer = None

        for conformer in conformers:
            name = conformer.split('_DFT')[0]
            conformer_log = f'{name}.log'

            conformer_info = dft_info_df[dft_info_df['file'] == conformer_log]

            if not conformer_info.empty:
                energy = conformer_info.iloc[0]['E']
                if energy < lowest_energy:
                    lowest_energy = energy
                    lowest_energy_conformer = conformer

            lowest_energy_conformer_xyz = f'{lowest_energy_conformer}.xyz'
            lowest_energy_conformer_dir = os.path.join(base_dir, lowest_energy_conformer_xyz)

        for conformer in conformers:
            ligand = conformer.split('_')[0]
            if conformer == lowest_energy_conformer:
                # rows.append((ligand, lowest_energy_conformer, conformer, 0.0))
                rows.append((ligand, coord, lowest_energy_conformer, conformer, 0.0))
            elif conformer != lowest_energy_conformer:
                conformer_xyz = f'{conformer}.xyz'
                conformer_dir = os.path.join(base_dir, conformer_xyz)

                command = f'calculate_rmsd --no-hydrogen {lowest_energy_conformer_dir} {conformer_dir}'
                result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE, text=True)
                rmsd_value = float(result.stdout.split('\n')[0])

                rows.append((ligand, coord, lowest_energy_conformer, conformer, rmsd_value))

        # rmsd_df = pd.DataFrame(rows, columns=['Ligand', 'lowest_DFT', 'conformer_DFT', 'RMSD_value'])
        rmsd_df = pd.DataFrame(rows, columns=['Ligand', 'Coordination_type', 'lowest_DFT', 'conformer_DFT', 'RMSD_value'])
        sorted_df = rmsd_df.sort_values(by=['Ligand', 'lowest_DFT', 'RMSD_value'])
        output_path = os.path.join(base_dir, output_filename)
        sorted_df.to_csv(output_path, index=False)

def calculate_complex_statistics(ligand_rmsd_file, substrate_rmsd_file, output_file):
    # Load the data from ligand_rmsd.csv and substrate_rmsd.csv
    ligand_rmsd_df = pd.read_csv(ligand_rmsd_file)
    substrate_rmsd_df = pd.read_csv(substrate_rmsd_file)

    # Filter out lowest energy conformers
    ligand_rmsd_df = ligand_rmsd_df[(ligand_rmsd_df['RMSD_value'] != 0.0)]
    substrate_rmsd_df = substrate_rmsd_df[(substrate_rmsd_df['RMSD_value'] != 0.0)]

    # Calculate mean and standard deviation for ligand RMSD
    ligand_stats = ligand_rmsd_df.groupby(['Ligand', 'Coordination_type'])['RMSD_value'].agg(['mean', 'std']).reset_index()
    ligand_stats = ligand_stats.rename(columns={'mean': 'ligand_mean', 'std': 'ligand_std'})

    # Calculate mean and standard deviation for substrate RMSD
    substrate_stats = substrate_rmsd_df.groupby(['Ligand', 'Coordination_type'])['RMSD_value'].agg(['mean', 'std']).reset_index()
    substrate_stats = substrate_stats.rename(columns={'mean': 'substrate_mean', 'std': 'substrate_std'})

    # Merge ligand and substrate statistics
    combined_stats = ligand_stats.merge(substrate_stats, on=['Ligand', 'Coordination_type'], how='outer').fillna(0)

    # Save the combined statistics to the output CSV file
    combined_stats.to_csv(output_file, index=False)

# Usage for substrate:
conformers_dir = './results/SM2_all' # Directory where all the L-Rh-S conformers can be found
isolate_ligand = os.path.join(os.getcwd(), 'results', 'isolate_ligand')
isolate_substrate = os.path.join(os.getcwd(), 'results', 'isolate_substrate')
files = os.listdir(conformers_dir)
for file in files:
    if file.endswith('_DFT.xyz'):
        xyz_filename = os.path.join(conformers_dir, file)
        save_only_metal_ligand_complex_in_xyz(xyz_filename, isolate_ligand)
        save_shortest_ligand_complex_in_xyz(xyz_filename, isolate_substrate)

calculate_rmsd_and_create_csv(isolate_ligand, 'L-Rh-S-ligand_rmsd.csv', 'L-Rh-S-conformer_info.csv', ',')
calculate_rmsd_and_create_csv(isolate_substrate, 'L-Rh-S-substrate_rmsd.csv', 'L-Rh-S-conformer_info.csv', ',')

ligand_rmsd_file = 'L-Rh-S-ligand_rmsd.csv'
substrate_rmsd_file = 'L-Rh-S-substrate_rmsd.csv'

calculate_complex_statistics(ligand_rmsd_file, substrate_rmsd_file, 'L-Rh-S-isolate_stats.csv')

# Usage for NBD:
conformers_dir = './results/NBD_all' # Directory where all L-Rh-NBD conformers can be found
isolate_ligand = os.path.join(os.getcwd(), 'results', 'NBD_isolate_ligand')
isolate_substrate = os.path.join(os.getcwd(), 'results', 'NBD_isolate_substrate')
files = os.listdir(conformers_dir)
for file in files:
    if file.endswith('_DFT.xyz'):
        xyz_filename = os.path.join(conformers_dir, file)
        save_only_metal_ligand_complex_in_xyz(xyz_filename, isolate_ligand)
        save_shortest_ligand_complex_in_xyz(xyz_filename, isolate_substrate)

calculate_rmsd_and_create_csv(isolate_ligand, 'L-Rh-NBD-ligand_rmsd.csv', 'L-Rh-NBD-conformer_info.csv', ',')
calculate_rmsd_and_create_csv(isolate_substrate, 'L-Rh-NBD-substrate_rmsd.csv', 'L-Rh-NBD-conformer_info.csv', ',')

ligand_rmsd_file = 'L-Rh-NBD-ligand_rmsd.csv'
substrate_rmsd_file = 'L-Rh-NBD-substrate_rmsd.csv'

calculate_complex_statistics(ligand_rmsd_file, substrate_rmsd_file, 'L-Rh-NBD_isolate_stats.csv')

############# Begin plotting part ########################
PROPS = {
    'boxprops':{'edgecolor':'black'},
    'medianprops':{'color':'black'},
    'whiskerprops':{'color':'black'},
    'capprops':{'color':'black'}}
palette = sns.color_palette('Paired')

# Read the data
ligand_data = pd.read_csv('L-Rh-S-ligand_rmsd.csv')
ligand_data = ligand_data[(ligand_data['RMSD_value'] != 0.0)]

substrate_data = pd.read_csv('L-Rh-S-substrate_rmsd.csv')
substrate_data = substrate_data[(substrate_data['RMSD_value'] != 0.0)]

# Add a 'Type' column to distinguish ligand and substrate data
ligand_data['Type'] = 'Ligand'
substrate_data['Type'] = 'Substrate'

# Combine the data into one DataFrame
combined_data = pd.concat([ligand_data, substrate_data])

# Mapping dictionary for ligand names, from original experimental names to manuscript names
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'Ligand' column
combined_data['Ligand'] = combined_data['Ligand'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Sort DataFrame based on ligand order
combined_data = combined_data.set_index('Ligand').loc[ligand_order].reset_index()

# Separate min and maj
maj_data = combined_data[combined_data['Coordination_type'] == 'maj']
min_data = combined_data[combined_data['Coordination_type'] == 'min']

##### Plot for major ##########
# Set the style for the plot
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

# Create a figure and axis
plt.subplots(figsize=(10, 6))

# Create the boxplot for major data
sns.boxplot(x='Ligand', y='RMSD_value', data=maj_data, hue='Type', palette={'Ligand': palette[5], 'Substrate': palette[4]}, width=0.5, **PROPS)
plt.grid(True)

# Set labels and title
plt.xlabel('Ligand')
plt.ylabel('RMSD (Å)')
plt.ylim(top=6)
plt.legend()
plt.savefig('./plots/maj_isolate_stats_boxplot.png', dpi=500, bbox_inches='tight')
plt.show()

###### Plot for minor ########
# Set the style for the plot
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

# Create a figure and axis
plt.subplots(figsize=(10, 6))

# Create the boxplot for minor data
sns.boxplot(x='Ligand', y='RMSD_value', data=min_data, hue='Type', palette={'Ligand': palette[1], 'Substrate': palette[0]}, width=0.5, **PROPS)
plt.grid(True)

# Set labels and title
plt.xlabel('Ligand')
plt.ylabel('RMSD (Å)')
plt.ylim(top=6)
plt.legend()
plt.savefig('./plots/min_isolate_stats_boxplot.png', dpi=500, bbox_inches='tight')
plt.show()


########## Plot for L-Rh-NBD ##########
PROPS = {
    'boxprops':{'edgecolor':'black'},
    'medianprops':{'color':'black'},
    'whiskerprops':{'color':'black'},
    'capprops':{'color':'black'}}
palette = sns.color_palette('Paired')
# Read the data
nbd_ligand_data = pd.read_csv('L-Rh-NBD-ligand_rmsd.csv')
nbd_ligand_data = nbd_ligand_data[(nbd_ligand_data['RMSD_value'] != 0.0)]
nbd_substrate_data = pd.read_csv('L-Rh-NBD-substrate_rmsd.csv')
nbd_substrate_data = nbd_substrate_data[(nbd_substrate_data['RMSD_value'] != 0.0)]

# Add a 'Type' column to distinguish ligand and substrate data
nbd_ligand_data['Type'] = 'Ligand'
nbd_substrate_data['Type'] = 'Substrate'

# Combine the data into one DataFrame
combined_data = pd.concat([nbd_ligand_data, nbd_substrate_data])

# Mapping dictionary for ligand names, from original experimental names to manuscript names
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
combined_data['Ligand'] = combined_data['Ligand'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L7', 'L8', 'L9', 'L11']

# Sort DataFrame based on ligand order
combined_data = combined_data.set_index('Ligand').loc[ligand_order].reset_index()

# Set the style for the plot
sns.set(style="whitegrid")
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14,  # Font size for legend
})

# Create a figure and axis
fig, ax = plt.subplots(figsize=(8, 6))

# Create the boxplot
sns.boxplot(x='Ligand', y='RMSD_value', data=combined_data, hue='Type', palette={'Ligand': palette[3], 'Substrate': palette[2]}, width=0.5, ax=ax, **PROPS)
plt.grid(True)

# Set labels and title
plt.xlabel('Ligand')
plt.ylabel('RMSD (Å)')
plt.ylim(top=6)
plt.legend()
plt.savefig('./plots/NBD_isolate_stats_boxplot.png', dpi=500, bbox_inches='tight')
plt.show()