# Setup of script: first, descriptors are calculated on .log files of all DFT-optimized L-Rh-S conformers.
# Then, the calculated descriptors are Boltzmann-averaged. Here, only five descriptors are chosen.
# The Boltzmann-averaging goes as follows: the two L-Rh-S major conformer ensembles are taken together, as are the two
# L-Rh-S minor conformer ensembles. Among all the conformers, the lowest energy conformer is found as reference point
# to calculate the Boltzmann weight. According to these weights, the descriptor values are averaged.
# Standard deviation and variance values are also calculated, and all put in a .csv file.
# These steps are first performed on L-Rh-S, followed by L-Rh-NBD and plotting.

from obelix.descriptor_calculator import Descriptors # import the workflow class if this you are not running code in run_workflow.py
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns

# Descriptor calculation for log files
dft_descriptors = Descriptors(central_atom='Rh', path_to_workflow=os.path.join(os.getcwd(), 'results','SM2_all'),
                              output_type='gaussian')
dft_descriptors.calculate_dft_descriptors_from_log(geom_type='BD', solvent=None, extract_xyz_from_log=True,
                                                   printout=False, metal_adduct='pristine', plot_steric_map=False)
dft_descriptors.descriptor_df.to_csv('L-Rh-S_DFT_descriptors.csv', index=False)

df = pd.read_csv('L-Rh-S_DFT_descriptors.csv')

df['ComplexName'] = df['filename_tud'].apply(lambda x: x.split('_')[0])
df['coord'] = df['filename_tud'].apply(lambda x: 'min' if 'min' in x else 'maj')

# Insert 'ComplexName' at the beginning (column 1)
df.insert(0, 'ComplexName', df.pop('ComplexName'))

# Insert 'coord' at the beginning (column 2)
df.insert(1, 'coord', df.pop('coord'))

# Sort the DataFrame based on 'ComplexName' and 'coord'
df.sort_values(by=['ComplexName', 'coord', 'filename_tud'], inplace=True, ignore_index=True)

# Save or use the modified DataFrame as needed
df.to_csv('L-Rh-S_DFT_descriptors.csv', index=False)

############ Boltzmann average descriptors #########
descriptor_df = pd.read_csv('L-Rh-S_DFT_descriptors.csv')
dft_info_df = pd.read_csv('L-Rh-S-conformer_info.csv', delimiter=',')

# Separate conformers into major/minor groups for each ligand, with the Bolzmann weight.
dict = {}
k = 8.617333262145e-5  # Boltzmann constant in eV/K
T = 298.15 # Temperature in Kelvin

for file in descriptor_df['filename_tud']:
    ligand_name = file.split('_')[0]
    if 'min' in file:
        basename = f'{ligand_name}_min'
    else:
        basename = ligand_name

    if not basename in dict:
        dict[basename] = []

    # Store E value from log file
    log_file = f'{file}.log'
    conformer_info = dft_info_df[dft_info_df['file'] == log_file]
    E_hartree = conformer_info.iloc[0]['E'] # E in hartree

    dict[basename].append({file: E_hartree})

# Store Boltzmann weight
update_dict = {}

for base, conformers in dict.items():
    update_conf = []
    energies = np.array([list(conformer.values())[0] for conformer in conformers])

    for conformer in conformers:
        E_rel = list(conformer.values())[0] - np.min(energies)
        E_ev = E_rel * 27.21138602 # value in eV
        boltzmann_weight = np.exp(-(E_ev)/(k * T))
        update_conf.append({list(conformer.keys())[0]: boltzmann_weight})

    update_dict[base] = update_conf

# Boltzmann average for 5 descriptors: HOMO/LUMO, Rh burried volume, donor buried volume, NBO Rh, NBO donor.
# Note that NBO donor and buried volume donor is an average of donor min and donor max, i.e., the two donor atoms.
rows = []

for base, conformers in update_dict.items():
    coord = 'min' if 'min' in base else 'maj'
    base = base.split('_')[0]

    base_homo_lumo = []
    base_bur_vol_rh = []
    base_nbo_rh = []
    base_bur_vol_donor = []
    base_nbo_donor = []

    base_weights = []

    for conformer in conformers:
        for name, boltzmann in conformer.items():
            descriptor_info = descriptor_df[descriptor_df['filename_tud'] == name]
            homo_lumo_dft = descriptor_info.iloc[0]['homo_lumo_gap_dft']
            bur_vol_rh = descriptor_info.iloc[0]['buried_volume_Rh_3.5A']
            nbo_rh = descriptor_info.iloc[0]['nbo_charge_Rh_dft']
            bur_vol_donor = np.average(
                (descriptor_info.iloc[0]['buried_volume_donor_max'], descriptor_info.iloc[0]['buried_volume_donor_min']))
            nbo_donor = np.average(
                (descriptor_info.iloc[0]['nbo_charge_min_donor_dft'], descriptor_info.iloc[0]['nbo_charge_max_donor_dft']))

            base_homo_lumo.append(homo_lumo_dft)
            base_bur_vol_rh.append(bur_vol_rh)
            base_nbo_rh.append(nbo_rh)
            base_bur_vol_donor.append(bur_vol_donor)
            base_nbo_donor.append(nbo_donor)

            base_weights.append(boltzmann)

############# Store L-Rh-S Boltzmann average, std and variance of descriptors ############
    # Global descriptor: HOMO/LUMO
    homo_lumo_avg = np.average(base_homo_lumo, weights=base_weights)
    homo_lumo_std = np.sqrt(np.average((base_homo_lumo - homo_lumo_avg) ** 2, weights=base_weights))
    homo_lumo_var = np.average((base_homo_lumo - homo_lumo_avg) ** 2, weights=base_weights)

    # Local descriptor: Rh buried volume
    bur_vol_rh_avg = np.average(base_bur_vol_rh, weights=base_weights)
    bur_vol_rh_std = np.sqrt(np.average((base_bur_vol_rh - bur_vol_rh_avg) ** 2, weights=base_weights))
    bur_vol_rh_var = np.average((base_bur_vol_rh - bur_vol_rh_avg) ** 2, weights=base_weights)

    # Local descriptor: Rh NBO charge
    nbo_rh_avg = np.average(base_nbo_rh, weights=base_weights)
    nbo_rh_std = np.sqrt(np.average((base_nbo_rh - nbo_rh_avg) ** 2, weights=base_weights))
    nbo_rh_var = np.average((base_nbo_rh - nbo_rh_avg) ** 2, weights=base_weights)

    # Local descriptor: donor buried volume
    bur_vol_donor_avg = np.average(base_bur_vol_donor, weights=base_weights)
    bur_vol_donor_std = np.sqrt(np.average((base_bur_vol_donor - bur_vol_donor_avg) ** 2, weights=base_weights))
    bur_vol_donor_var = np.average((base_bur_vol_donor - bur_vol_donor_avg) ** 2, weights=base_weights)

    # Local descriptor: donor NBO charge
    nbo_donor_avg = np.average(base_nbo_donor, weights=base_weights)
    nbo_donor_std = np.sqrt(np.average((base_nbo_donor - nbo_donor_avg) ** 2, weights=base_weights))
    nbo_donor_var = np.average((base_nbo_donor - nbo_donor_avg) ** 2, weights=base_weights)

    rows.append([base, coord, homo_lumo_avg, homo_lumo_std, homo_lumo_var, bur_vol_rh_avg, bur_vol_rh_std, bur_vol_rh_var,
                 nbo_rh_avg, nbo_rh_std, nbo_rh_var, bur_vol_donor_avg, bur_vol_donor_std, bur_vol_donor_var,
                 nbo_donor_avg, nbo_donor_std, nbo_donor_var])

# Create CSV file
avg_df = pd.DataFrame(rows, columns=['Ligand', 'Coordination_type', 'homo_lumo_dft_avg', 'homo_lumo_dft_std', 'homo_lumo_dft_var',
                                     'bur_vol_Rh_avg', 'bur_vol_Rh_std', 'bur_vol_Rh_var', 'nbo_rh_avg',
                                     'nbo_rh_std', 'nbo_rh_var', 'bur_vol_donor_avg', 'bur_vol_donor_std', 'bur_vol_donor_var',
                                     'nbo_donor_avg', 'nbo_donor_std', 'nbo_donor_var'])
sorted_df = avg_df.sort_values(by=['Ligand'])
sorted_df.to_csv('L-Rh-S_DFT_descriptor_avg.csv', index=False)


###############################################################
################## For NBD ####################################
# example descriptor calculation for log files
dft_descriptors = Descriptors(central_atom='Rh', path_to_workflow=os.path.join(os.getcwd(), 'results',
                                                                               'NBD_all'),
                              output_type='gaussian')
dft_descriptors.calculate_dft_descriptors_from_log(geom_type='BD', solvent=None, extract_xyz_from_log=True,
                                                   printout=False, metal_adduct='pristine', plot_steric_map=False)
dft_descriptors.descriptor_df.to_csv('L-Rh-NBD_DFT_descriptors.csv', index=False)

############ Boltzmann average descriptors #########
descriptor_df = pd.read_csv('L-Rh-NBD_DFT_descriptors.csv')
dft_info_df = pd.read_csv('L-Rh-NBD-conformer_info.csv', delimiter=',')

# Separate conformers into major/minor groups for each ligand, with the Bolzmann weight.
dict = {}
k = 8.617333262145e-5  # Boltzmann constant in eV/K
T = 298.15 # Temperature in Kelvin

for file in descriptor_df['filename_tud']:
    ligand_name = file.split('_')[0]

    if not ligand_name in dict:
        dict[ligand_name] = []

    # Store E value from log file
    log_file = f'{file}.log'
    conformer_info = dft_info_df[dft_info_df['file'] == log_file]
    E_hartree = conformer_info.iloc[0]['E'] # E in hartree

    dict[ligand_name].append({file: E_hartree})

# Store Boltzmann weight
update_dict = {}

for base, conformers in dict.items():
    update_conf = []
    energies = np.array([list(conformer.values())[0] for conformer in conformers])

    for conformer in conformers:
        E_rel = list(conformer.values())[0] - np.min(energies)
        E_ev = E_rel * 27.21138602 # value in eV
        boltzmann_weight = np.exp(-(E_ev)/(k * T))
        update_conf.append({list(conformer.keys())[0]: boltzmann_weight})

    update_dict[base] = update_conf

# Boltzmann average for 5 descriptors: HOMO/LUMO, Rh burried volume, donor buried volume, NBO Rh, NBO donor.
# Note that NBO donor and buried volume donor is an average of donor min and donor max, i.e., the two donor atoms.
rows = []

for base, conformers in update_dict.items():
    base = base.split('_')[0]

    base_homo_lumo = []
    base_bur_vol_rh = []
    base_nbo_rh = []
    base_bur_vol_donor = []
    base_nbo_donor = []

    base_weights = []

    for conformer in conformers:
        for name, boltzmann in conformer.items():
            descriptor_info = descriptor_df[descriptor_df['filename_tud'] == name]
            homo_lumo_dft = descriptor_info.iloc[0]['homo_lumo_gap_dft']
            bur_vol_rh = descriptor_info.iloc[0]['buried_volume_Rh_3.5A']
            nbo_rh = descriptor_info.iloc[0]['nbo_charge_Rh_dft']
            bur_vol_donor = np.average(
                (descriptor_info.iloc[0]['buried_volume_donor_max'], descriptor_info.iloc[0]['buried_volume_donor_min']))
            nbo_donor = np.average(
                (descriptor_info.iloc[0]['nbo_charge_min_donor_dft'], descriptor_info.iloc[0]['nbo_charge_max_donor_dft']))

            base_homo_lumo.append(homo_lumo_dft)
            base_bur_vol_rh.append(bur_vol_rh)
            base_nbo_rh.append(nbo_rh)
            base_bur_vol_donor.append(bur_vol_donor)
            base_nbo_donor.append(nbo_donor)

            base_weights.append(boltzmann)

############# Store L-Rh-NBD Boltzmann average, std and variance of descriptors ############
    # Global descriptor: HOMO/LUMO
    homo_lumo_avg = np.average(base_homo_lumo, weights=base_weights)
    homo_lumo_std = np.sqrt(np.average((base_homo_lumo - homo_lumo_avg) ** 2, weights=base_weights))
    homo_lumo_var = np.average((base_homo_lumo - homo_lumo_avg) ** 2, weights=base_weights)

    # Local descriptor: Rh buried volume
    bur_vol_rh_avg = np.average(base_bur_vol_rh, weights=base_weights)
    bur_vol_rh_std = np.sqrt(np.average((base_bur_vol_rh - bur_vol_rh_avg) ** 2, weights=base_weights))
    bur_vol_rh_var = np.average((base_bur_vol_rh - bur_vol_rh_avg) ** 2, weights=base_weights)

    # Local descriptor: Rh NBO charge
    nbo_rh_avg = np.average(base_nbo_rh, weights=base_weights)
    nbo_rh_std = np.sqrt(np.average((base_nbo_rh - nbo_rh_avg) ** 2, weights=base_weights))
    nbo_rh_var = np.average((base_nbo_rh - nbo_rh_avg) ** 2, weights=base_weights)

    # Local descriptor: donor buried volume
    bur_vol_donor_avg = np.average(base_bur_vol_donor, weights=base_weights)
    bur_vol_donor_std = np.sqrt(np.average((base_bur_vol_donor - bur_vol_donor_avg) ** 2, weights=base_weights))
    bur_vol_donor_var = np.average((base_bur_vol_donor - bur_vol_donor_avg) ** 2, weights=base_weights)

    # Local descriptor: donor NBO charge
    nbo_donor_avg = np.average(base_nbo_donor, weights=base_weights)
    nbo_donor_std = np.sqrt(np.average((base_nbo_donor - nbo_donor_avg) ** 2, weights=base_weights))
    nbo_donor_var = np.average((base_nbo_donor - nbo_donor_avg) ** 2, weights=base_weights)

    rows.append([base, homo_lumo_avg, homo_lumo_std, homo_lumo_var, bur_vol_rh_avg, bur_vol_rh_std, bur_vol_rh_var,
                 nbo_rh_avg, nbo_rh_std, nbo_rh_var, bur_vol_donor_avg, bur_vol_donor_std, bur_vol_donor_var,
                 nbo_donor_avg, nbo_donor_std, nbo_donor_var])

# Create CSV file
avg_df = pd.DataFrame(rows, columns=['Ligand', 'homo_lumo_dft_avg', 'homo_lumo_dft_std', 'homo_lumo_dft_var',
                                     'bur_vol_Rh_avg', 'bur_vol_Rh_std', 'bur_vol_Rh_var', 'nbo_rh_avg',
                                     'nbo_rh_std', 'nbo_rh_var', 'bur_vol_donor_avg', 'bur_vol_donor_std', 'bur_vol_donor_var',
                                     'nbo_donor_avg', 'nbo_donor_std', 'nbo_donor_var'])
sorted_df = avg_df.sort_values(by=['Ligand'])
sorted_df.to_csv('L-Rh-NBD_DFT_descriptors_avg.csv', index=False)


######### Begin plotting part #####################
# Set the plot style
sns.set(style="whitegrid")
palette = sns.color_palette('tab10')
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 13,  # Font size for legend
})

# Load the CSV data
data_NBD = pd.read_csv('L-Rh-NBD_DFT_descriptors_avg.csv')
data_SM2 = pd.read_csv('L-Rh-S_DFT_descriptor_avg.csv')

# Extract ligands and descriptors
ligands = data_SM2['Ligand']
descriptors = data_SM2.columns[2::3]  # Assuming every third column is a descriptor column
num_descriptors = len(descriptors)

# Mapping dictionary for ligand names
ligand_mapping = {'L33': 'L1', 'L40': 'L2', 'L41': 'L3', 'L42': 'L4', 'L46': 'L5',
                  'L47': 'L6', 'L48': 'L7', 'L49': 'L8', 'L50': 'L9', 'L89': 'L10', 'L93': 'L11'}

# Apply the mapping to the 'ComplexName' column
data_SM2['Ligand'] = data_SM2['Ligand'].map(ligand_mapping)
data_NBD['Ligand'] = data_NBD['Ligand'].map(ligand_mapping)

# Specify the order of ligands
ligand_order = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'L9', 'L10', 'L11']

# Sort DataFrame based on ligand order
data_SM2 = data_SM2.set_index('Ligand').loc[ligand_order].reset_index()
data_NBD = data_NBD.set_index('Ligand').loc[ligand_order].reset_index()

# Create separate plots for each descriptor group (average, std, and var)
for i in range(0, 5):
    descriptor_name = descriptors[i].split('_avg')[0]

    # Create a new figure and axis for this descriptor group
    fig, ax = plt.subplots(figsize=(8, 6))

    average_col = descriptors[i]
    std_col = average_col.replace("_avg", "_std")

    # Separate data based on 'coordination type'
    maj_data = data_SM2[data_SM2['Coordination_type'] == 'maj']
    min_data = data_SM2[data_SM2['Coordination_type'] == 'min']

    # Get the x-axis labels
    maj_labels = maj_data['Ligand']
    min_labels = min_data['Ligand']
    NBD_labels = data_NBD['Ligand']

    # Calculate the mean and standard deviation for each descriptor
    maj_means = maj_data[average_col]
    maj_stds = maj_data[std_col]

    min_means = min_data[average_col]
    min_stds = min_data[std_col]

    nbd_means = data_NBD[average_col]
    nbd_stds = data_NBD[std_col]

    # Create positions for the major and minor data points
    maj_positions = np.arange(len(maj_labels))
    min_positions = np.arange(len(maj_labels))
    nbd_positions = np.arange(len(NBD_labels))

    # Set the x-axis ticks in the middle of major and minor positions
    x_positions = np.concatenate((maj_positions, min_positions))
    x_labels = np.concatenate((maj_data['Ligand'], min_data['Ligand']))
    plt.xticks(x_positions, x_labels)

    # Plot error bars for 'coordination type' = 'min' in another color
    plt.errorbar(min_positions -0.15, min_means, yerr=min_stds, label='L-Rh-S minor', marker='o', linestyle='none', ms=5, capsize=5)

    # Plot error bars for 'coordination type' = 'maj' in one color
    plt.errorbar(maj_positions, maj_means, yerr=maj_stds, label='L-Rh-S major', marker='o', linestyle='none', ms=5, capsize=5, ecolor=palette[3], mfc=palette[3], mec=palette[3])

    # Plot error bars for NBD
    plt.errorbar(nbd_positions + 0.15, nbd_means, yerr=nbd_stds, label='L-Rh-NBD', marker='o', linestyle='none', ms=5, capsize=5,
                 ecolor=palette[2], mfc=palette[2], mec=palette[2])

    # Set labels and title
    plt.xlabel('Ligand')
    plt.ylabel(f'{descriptor_name}')
    #plt.ylabel('averaged NBO charge donor atoms (a.u.)')
    #plt.ylim(1.050, 1.250)
    #plt.title(f'Average with Error Whiskers for {descriptor_name}')

    # Place the legend outside of the plot to the right
    plt.legend()

    # Save the plot to a file
    plt.savefig(f'./plots/together_{descriptor_name}_descriptor_stats_all.png', dpi=500, bbox_inches='tight')
    plt.show()
