# Script setup: In the function that is defines, two conformers are structurally compared to one another.
# This is done by reading the .csv files with conformer RMSD values. The files that are needed are:
# E vs RMSD (from Figure 4), ligand RMSD, and substrate RMSD (both from Figure 6).
# The function first extracts the RMSD values of the two conformers that are given as input. Then the difference in
# total RMSD, energy value, substrate RMSD, and ligand RMSD is stored. Then, the maximum ligand and substrate RMSD value
# for the given ligands is found. This is needed to write the final ligand and substrate RMSD differences as
# percentage of the largest ligand/substrate RMSD value. This is done because the range of ligand RMSD values is
# significantly larger than the range of substrate RMSD values.
# The idea is to give as input two conformers that are 1) different in total RMSD with constant energy (horizintal
# overlay), or 2) different in energy with the same RMSD value (vertical overlay). After a few of these instances
# are given, the results are written in a .csv file, and plotted separately for the horizontal and vertical overlays,
# to see which part of the catalyst (ligand or substrate) contributes more to total RMSD or energy differences.

import pandas as pd
import os
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np

def process_csv_files(overlay, conformer1, conformer2, e_rmsd_file, ligand_file, substrate_file, output_file):
    # Check if the output file already exists
    if os.path.exists(output_file):
        # Read existing data
        existing_data = pd.read_csv(output_file)

        # Check if the conformer pair already exists in the output file
        if any((existing_data['conformer1'] == conformer1) & (existing_data['conformer2'] == conformer2)):
            print(f"The conformer pair {conformer1} - {conformer2} already exists in the output file.")
            return

    else:
        # If the file doesn't exist, initialize an empty DataFrame
        existing_data = pd.DataFrame()

    # Read CSV files
    e_rmsd_data = pd.read_csv(e_rmsd_file)
    ligand_data = pd.read_csv(ligand_file)
    substrate_data = pd.read_csv(substrate_file)

    # Extract values for conformer1
    conformer1_e_rmsd_values = e_rmsd_data[e_rmsd_data['other'] == conformer1]
    conformer1_ligand_values = ligand_data[ligand_data['conformer_DFT'].str.startswith(conformer1)]
    conformer1_substrate_values = substrate_data[substrate_data['conformer_DFT'].str.startswith(conformer1)]

    # Extract values for conformer2
    conformer2_e_rmsd_values = e_rmsd_data[e_rmsd_data['other'] == conformer2]
    conformer2_ligand_values = ligand_data[ligand_data['conformer_DFT'].str.startswith(conformer2)]
    conformer2_substrate_values = substrate_data[substrate_data['conformer_DFT'].str.startswith(conformer2)]

    # Calculate absolute differences
    e_dif_diff = abs(conformer1_e_rmsd_values.loc[:, 'E_dif_(kJ/mol)'].values[0] -
                     conformer2_e_rmsd_values.loc[:, 'E_dif_(kJ/mol)'].values[0])

    rmsd_value_diff = abs(conformer1_e_rmsd_values.loc[:, 'rmsd_value_DFT'].values[0] -
                          conformer2_e_rmsd_values.loc[:, 'rmsd_value_DFT'].values[0])

    # Calculate percentage change
    max_ligand_rmsd = ligand_data.groupby(ligand_data['conformer_DFT'].str.split('_conformer').str[0])[
        'RMSD_value'].max().loc[conformer1.split('_conformer')[0]]

    max_substrate_rmsd = substrate_data.groupby(substrate_data['conformer_DFT'].str.split('_conformer').str[0])[
        'RMSD_value'].max().loc[conformer1.split('_conformer')[0]]

    ligand_percentage = abs(conformer1_ligand_values.loc[:, 'RMSD_value'].values[0] -
                            conformer2_ligand_values.loc[:, 'RMSD_value'].values[0]) / max_ligand_rmsd * 100

    substrate_percentage = abs(conformer1_substrate_values.loc[:, 'RMSD_value'].values[0] -
                               conformer2_substrate_values.loc[:, 'RMSD_value'].values[0]) / max_substrate_rmsd * 100

    # Create the final DataFrame
    new_data = pd.DataFrame({
        'overlay': overlay,
        'conformer1': [conformer1],
        'conformer2': [conformer2],
        'rmsd_value_DFT_diff': [rmsd_value_diff],
        'E_dif_diff': [e_dif_diff],
        'ligand_percentage_change': ligand_percentage,
        'substrate_percentage_change': substrate_percentage
    })

    # Combine with existing data, if any
    combined_data = pd.concat([existing_data, new_data], ignore_index=True)

    # Sort the DataFrame
    combined_data.sort_values(by=['overlay', 'conformer1'], inplace=True)

    # Write the combined DataFrame to the output CSV file
    combined_data.to_csv(output_file, index=False)

# Example usage
e_rmsd = 'E_vs_RMSD_DFT_CREST.csv'
ligand = 'L-Rh-S-ligand_rmsd.csv'
substrate = 'L-Rh-S-substrate_rmsd.csv'

#process_csv_files('-', 'L41_min_conformer_6', 'L41_min_conformer_47', e_rmsd, ligand, substrate, 'ligand_substrate_difference.csv')

################# Begin plotting part ##########################
# Set the plot style
sns.set(style="whitegrid")
palette = sns.color_palette('Paired')
plt.rcParams.update({
    'font.size': 14,       # Default font size for text
    'axes.labelsize': 16,  # Font size for axis labels
    'axes.titlesize': 16,  # Font size for plot title
    'xtick.labelsize': 14,  # Font size for x-axis tick labels
    'ytick.labelsize': 14,  # Font size for y-axis tick labels
    'legend.fontsize': 14, # Font size for legend
})

df = pd.read_csv('ligand_substrate_difference.csv')

# Filter data for 'hor' and 'vert' overlay
hor_data = df[df['overlay'] == 'hor']
vert_data = df[df['overlay'] == 'vert']

# Extract ligand and conf1 from conformer1, and conf2 from conformer2
hor_data['ligand'] = hor_data['conformer1'].apply(lambda x: x.split('_')[0])
hor_data['conf1'] = hor_data['conformer1'].apply(lambda x: x.split('_')[-1])
hor_data['conf2'] = hor_data['conformer2'].apply(lambda x: x.split('_')[-1])

vert_data['ligand'] = vert_data['conformer1'].apply(lambda x: x.split('_')[0])
vert_data['conf1'] = vert_data['conformer1'].apply(lambda x: x.split('_')[-1])
vert_data['conf2'] = vert_data['conformer2'].apply(lambda x: x.split('_')[-1])

# Create x-tick labels
xtick_labels_hor = hor_data['ligand'] + ', ' + hor_data['conf1'] + ' - ' + hor_data['conf2']
xtick_labels_vert = vert_data['ligand'] + ', ' + vert_data['conf1'] + ' - ' + vert_data['conf2']

# # Add 'min' or 'maj' to the label
# min_or_maj = ['min' if 'min' in conf1 else 'maj' for conf1 in hor_data['conformer1']]
# print(min_or_maj)
# xtick_labels += '\n' + pd.Series(min_or_maj).astype(str)

# Plot barplots for 'ligand_percentage_change' and 'substrate_percentage_change' HORIZONTAL
fig, ax = plt.subplots(figsize=(8, 6))
bar_width = 0.35
conformer_pairs = hor_data['conformer1'] + ' - ' + hor_data['conformer2']

bar1 = ax.bar(np.arange(len(conformer_pairs)), hor_data['ligand_percentage_change'], bar_width, label='Ligand', color=palette[9])
bar2 = ax.bar(np.arange(len(conformer_pairs)) + bar_width, hor_data['substrate_percentage_change'], bar_width, label='Substrate', color=palette[8])

ax.legend()
ax.set_xticks(np.arange(len(conformer_pairs)) + bar_width / 2)
ax.set_xticklabels(xtick_labels_hor, rotation=45, ha='right')  # Rotate x-tick labels for better visibility

plt.title('∆RMSD - ∆E constant')
plt.xlabel('Conformer Pairs')
plt.ylabel('Percentage Change')
plt.tight_layout()  # Adjust layout
plt.savefig('./plots/percent_RMSD_constant_E_correct', dpi=500, bbox_inches='tight')
plt.show()

# Plot barplots for 'ligand_percentage_change' and 'substrate_percentage_change' VERTICAL
fig_vert, ax = plt.subplots(figsize=(12, 6))
bar_width = 0.35
conformer_pairs = vert_data['conformer1'] + ' - ' + vert_data['conformer2']

bar1_vert = ax.bar(np.arange(len(conformer_pairs)), vert_data['ligand_percentage_change'], bar_width, label='Ligand', color=palette[9])
bar2_vert = ax.bar(np.arange(len(conformer_pairs)) + bar_width, vert_data['substrate_percentage_change'], bar_width, label='Substrate', color=palette[8])

ax.legend()
ax.set_xticks(np.arange(len(conformer_pairs)) + bar_width / 2)
ax.set_xticklabels(xtick_labels_vert, rotation=45, ha='right')  # Rotate x-tick labels for better visibility

plt.title('RMSD constant - ∆∆E')
plt.xlabel('Conformer Pairs')
plt.ylabel('Percentage Change')
plt.tight_layout()  # Adjust layout
plt.savefig('./plots/percent_E_constant_RMSD_correct', dpi=500, bbox_inches='tight')
plt.show()
